#!/usr/bin/python3
# ДАННЫЙ ПРИМЕР ВЫПОЛНЯЕТ КАЛИБРОВКУ ДЖОЙСТИКА:

from curses import wrapper
from time import time, sleep

# Подключаем библиотеку для работы с джойстиком I2C-flash.
from pyiArduinoI2Cjoystick import *

# Объявляем объект joy для работы с библиотекой, указывая адрес модуля на шине I2C.
j = pyiArduinoI2Cjoystick(0x09)

#   Если объявить объект без указания адреса, то адрес будет найден автоматически.
# j = pyiArduinoI2Cjoystick()


def nillis():
    return int(round(time() * 1000))

start = nillis()

def millis():
    return nillis() - start

x=0
adc_X_min=4096
adc_X_cen=0
adc_X_max=0

y=0
adc_Y_min=4096
adc_Y_cen=0
adc_Y_max=0

# Устанавливаем незначительное усреднение показаний АЦП.
j.setAveraging(2)

# Устанавливаем мёртвую зону центрального положения в 7%.
j.setDeadZone(7.0)

# Готовимся определить центральное положение джойстика:                         #
print()
print("КАЛИБРОВКА ДЖОЙСТИКА:")
print("Отпустите джойстик.")
print( "Он должен самостоятельно установиться в центральное положение.")

print("Ждём.", end='')

# Даём время пользователю отпустить джойстик.
for i in range(5):

    sleep(1)
    print(".", end='')

# Получаем значения АЦП соответствующие центральному положению:

# Выполняем цикл чтения 10 раз.
for i in range(10):

    x, y = j.getADC()
    print()

    # Получаем текущие значения АЦП в указанные переменные.
    print("АЦП = " + str(x) + ":" + str(y))

    # Получаем АЦП центра, прибавляя его к предыдущему значению.
    adc_X_cen += x
    adc_Y_cen += y

    sleep(.2)

# Получаем среднее арифметическое значение АЦП центра.

adc_X_cen /= 10
adc_Y_cen /= 10

# Готовимся определить крайние положения джойстика:

print("\r\nГотово!")
print("------------------------------------------------")
print("")
print("Теперь медленно поворачивайте джойстик по кругу,")
print("со скоростью 1 полный оборот за 2-3 секунды,")
print("на максимальном удалении джойстика от центра.", end='')

# Получаем значения АЦП соответствующие крайним положениям:

# Определяем время завершения снятия показаний.
tmrStop = millis() + 10000

# Выполняем цикл пока не достигнем времени tmrStop...
while millis() < tmrStop:

    # Получаем текущие значения АЦП в указанные переменные.
    x, y = j.getADC()

    # Получаем минимальные значение АЦП
    if x < adc_X_min: adc_X_min = x
    if y < adc_Y_min: adc_Y_min = y

    # Получаем максимальные значение АЦП по оси X.
    if x > adc_X_max: adc_X_max = x
    if y > adc_Y_max: adc_Y_max = y

    # Выводим точку в stdout, один раз в секунду.
    if millis() % 1000 < 10: sleep(.01),  print(".", end='')

# Выводим результат:

print("\r\nГотово!")
print("------------------------------------------------")
print( "")

print(
        "АЦП по оси X меняется от " + str(adc_X_min)
        + " до " + str(adc_X_max) + ", с центром в "
        + str(int(round(adc_X_cen)))
     )
print(
        "АЦП по оси Y меняется от " + str(adc_Y_min)
        + " до " + str(adc_Y_max) + ", с центром в "
        + str(int(round(adc_Y_cen)))
     )

print( "================================================")


# Калибруем джойстик (данные сохранятся в его flash памяти):

# Калибруем координаты по оси X.

j.setCalibration_X(adc_X_min, adc_X_cen, adc_X_max)

# Калибруем координаты по оси Y.
j.setCalibration_Y(adc_Y_min, adc_Y_cen, adc_Y_max)


# Готовимся к выводу координат после калибровки:

print("Приготовтесь проверить результат калибровки.")
print("Сейчас будут выводиться координаты джойстика...")
print("Ждём.", end = '')

# Даём время пользователю прочитать результат, до начала вывода координат.
for i in range(5):
    sleep(1)
    print(".", end='')

print("\r\nВыводим координаты:")


# Выводим координаты:
def output(stdscr):
    stdscr.clear()

    print("Нажмите <ctrl+c> для выхода", end="\r\n")
    x, y = j.getPosition()

    # Выводим координаты X:Y.
    print("X:Y="+str(x)+":"+str(y)+" ", end='' )
    x, y = j.getADC()

    # Выводим значения АЦП.
    print("(АЦП="+str(x)+":"+str(y)+")." )

    print()

    sleep(.1)

    stdscr.refresh()


while True:
    wrapper(output)



#!/usr/bin/python3
# ПРИМЕР РУЧНОЙ КАЛИБРОВКИ КООРДИНАТ ДЖОЙСТИКА:
"""
    Примечание:
    Значения АЦП у разных джойстиков могут отличаться
    Точные показания АЦП для каждого положения Вашего
    джойстика Вы можете узнать из примера «GetADC».

"""
from curses import wrapper
from time import sleep

#   Подключаем библиотеку для работы с джойстиком I2C-flash.
from pyiArduinoI2Cjoystick import *

# Объявляем объект для работы с библиотекой, указывая адрес модуля на шине I2C.
j = pyiArduinoI2Cjoystick(0x09)

#   Если объявить объект без указания адреса,
# то адрес будет найден автоматически.
#j = pyiArduinoI2Cjoystick()

#  Калибровка осей указанием значений АЦП:
#   Калибруем ось X указывая АЦП крайнего левого  положения (0),
# АЦП центра (2047), АЦП крайнего правого  положения (4095).
j.setCalibration_X(0, 2047, 4095)

#   Калибруем ось Y указывая АЦП крайнего нижнего положения (0),
# АЦП центра (2047), АЦП крайнего верхнего положения (4095).
j.setCalibration_Y(0, 2047, 4095)

#   Если направление оси требуется изменить,
# то поменяйте местами крайние значения АЦП оси (0 <-> 4095).

def main(stdscr):
    stdscr.clear()
    print("Текущие координаты: (нажмите <ctrl+c> для выхода)", end="\r\n")
    print( "X:Y=" + str(j.x) + ":" + str(j.y), end="\r\n")

    sleep(.1)
    stdscr.refresh()


while True:
    wrapper(main)



# distutils: language = c++
# cython: language_level = 3

from iarduino_I2C_Joystick cimport iarduino_I2C_Joystick
#from time import sleep

DEF_CHIP_ID_FLASH =  0x3C
DEF_CHIP_ID_METRO =  0xC3

# Адреса регистров модуля:
REG_FLAGS_0                  =  0x00
REG_BITS_0                   =  0x01
REG_FLAGS_1                  =  0x02
REG_BITS_1                   =  0x03
REG_MODEL                    =  0x04
REG_VERSION                  =  0x05
REG_ADDRESS                  =  0x06
REG_CHIP_ID                  =  0x07
REG_JOY_AVERAGING            =  0x08
REG_JOY_DEAD_ZONE            =  0x09
REG_JOY_KEY                  =  0x10
REG_JOY_KEY_TIM              =  0x11
REG_JOY_AXIS_X               =  0x12
REG_JOY_AXIS_Y               =  0x13
REG_JOY_X_ADC_L              =  0x14
REG_JOY_Y_ADC_L              =  0x16
REG_JOY_X_RIGHT_L            =  0x18
REG_JOY_X_CEN_L              =  0x1A
REG_JOY_X_LEFT_L             =  0x1C
REG_JOY_Y_UP_L               =  0x1E
REG_JOY_Y_CEN_L              =  0x20
REG_JOY_Y_DOWN_L             =  0x22
# Позиция битов и флагов:
JOY_KEY_PUSHED               =  0x80
JOY_KEY_RELEASED             =  0x40
JOY_KEY_PRESSED              =  0x20
JOY_KEY_TRIGGER              =  0x10
JOY_KEY_HOLD_05              =  0x08
JOY_KEY_HOLD_10              =  0x04
JOY_KEY_HOLD_20              =  0x02
JOY_KEY_CHANGED              =  0x01

KEY_PUSHED                   =  1
KEY_RELEASED                 =  2
KEY_PRESSED                  =  3
KEY_TRIGGER                  =  4
KEY_HOLD_05                  =  5
KEY_HOLD_10                  =  6
KEY_HOLD_20                  =  7
KEY_CHANGED                  =  8
KEY_TIME_PRESSED             =  9

NO_BEGIN = 1


cdef class pyiArduinoI2Cjoystick():
    cdef iarduino_I2C_Joystick c_module

    def __cinit__(self, address=None, auto=None):

        if address is not None:

            self.c_module = iarduino_I2C_Joystick(address)

            if auto is None:
                #sleep(.5)
                if not self.c_module.begin():

                    print("ошибка инициализации модуля.\n"
                          "Проверьте подключение и адрес модуля,"
                          " возможно не включен интерфейс I2C.\n"
                          " Запустите raspi-config и включите интерфейс"
                          ", инструкция по включению:"
                          " https://wiki.iarduino.ru/page/raspberry-i2c-spi/")

        else:

            self.c_module = iarduino_I2C_Joystick()

            if auto is None:
                #sleep(.5)
                if not self.c_module.begin():

                    print("ошибка инициализации модуля.\n"
                          "Проверьте подключение и адрес модуля, "
                          " возможно не включен интерфейс I2C.\n"
                          " Запустите raspi-config и включите интерфейс"
                          ", инструкция по включению:"
                          " https://wiki.iarduino.ru/page/raspberry-i2c-spi/")

    def begin(self):
        return self.c_module.begin()

    def changeAddress(self, unsigned char newAddr):
        return self.c_module.changeAddress(newAddr)

    def reset(self):
        return self.c_module.reset()

    def getAddress(self):
        return self.c_module.getAddress()

    def getVersion(self):
        return self.c_module.getVersion()

    def getPullI2C(self):
        return self.c_module.getPullI2C()

    def setPullI2C(self, f = True):
        return self.c_module.setPullI2C(f)

    def getButton(self, t):
        return self.c_module.getButton(t)

    def getPosition_X(self):
        return self.c_module.getPosition_X()

    def getPosition_Y(self):
        return self.c_module.getPosition_Y()

    def getADC_X(self):
        return self.c_module.getADC_X()

    def getADC_Y(self):
        return self.c_module.getADC_Y()

    def setCalibration_X(self, x_min, x_cen, x_max):
        return self.c_module.setCalibration_X(x_min, x_cen, x_max)

    def setCalibration_Y(self, y_min, y_cen, y_max):
        return self.c_module.setCalibration_Y(y_min, y_cen, y_max)

    def setDeadZone(self, zone):
        return self.c_module.setDeadZone(zone)

    def setAveraging(self, a):
        return self.c_module.setAveraging(a)

# now the fun part...

    def getPosition(self):
        nx = self.getPosition_X()
        ny = self.getPosition_Y()
        return nx, ny

    def getADC(self):
        nx = self.getADC_X()
        ny = self.getADC_Y()
        return nx, ny

    def getCalibration_X(self):
        self.c_module.updateCalX()
        l = self.c_module.getMinX()
        c = self.c_module.getCenX()
        r = self.c_module.getMaxX()
        return l, c, r

    def getCalibration_Y(self):
        self.c_module.updateCalY()
        d = self.c_module.getMinY()
        c = self.c_module.getCenY()
        u = self.c_module.getMaxY()
        return d, c, u

    def __getattr__(self, item):

        if item == 'x' or item == 'X':
            return self.c_module.getPosition_X()

        if item == 'y' or item == 'Y':
            return self.c_module.getPosition_Y()

        if item == 'ax' or item == 'AX':
            return self.c_module.getADC_X()

        if item == 'ay' or item == 'AY':
            return self.c_module.getADC_Y()



#!/usr/bin/python3
# ДАННЫЙ ПРИМЕР ВЫВОДИТ СЫРЫЕ ЗНАЧЕНИЯ АЦП:

from time import sleep
from curses import wrapper

# Подключаем библиотеку для работы с джойстиком I2C-flash.
from pyiArduinoI2Cjoystick import *

# Объявляем объект для работы с библиотекой, указывая адрес модуля на шине.
j = pyiArduinoI2Cjoystick(0x09)

# Если объявить объект без указания адреса, то адрес будет найден автоматически.
#j = pyiArduinoI2Cjoystick()


def main(stdscr):

    stdscr.clear()

    print("Текущие координаты: (нажмите <ctrl+c> для выхода)", end="\r\n")

    # Получаем и выводим АЦП по отдельности:

    print("X=", end='')

    # Выводим показания АЦП снятые с потенциометра установленного
    # на оси X джойстика, значение от 0 до 4095.
    print(j.ax, end='')

    print(", Y=", end='')

    # Выводим показания АЦП снятые с потенциометра установленного
    # на оси Y джойстика, значение от 0 до 4095.
    print(j.ay, end='')

    # Получаем сразу оба значения АЦП джойстика:
    x, y = j.getADC()

    print( ", X:Y=" + str(x) + ":" + str(y), end="\r\n")

    sleep(.1)

    stdscr.refresh()

while True:

    wrapper(main)



cdef extern from "iarduino_I2C_Joystick.cpp":
    pass

cdef extern from "iarduino_I2C_Joystick.h":
    cdef cppclass iarduino_I2C_Joystick:
        iarduino_I2C_Joystick() except +
        iarduino_I2C_Joystick(unsigned char) except +
        bint begin()
        bint changeAddress(unsigned char)
        bint reset()
        unsigned char getAddress()
        unsigned char getVersion()
        bint getPullI2C()
        bint setPullI2C(bint)
        unsigned short getButton(unsigned char)
        int getPosition_X()
        int getPosition_Y()
#template<typename T> bool getPosition(T &x, T &y)
        int getADC_X()
        int getADC_Y()
#template<typename T> bool getADC(T &x, T &y)
#template<typename T> bool getCalibration_X(T &l, T &c, T &r)
#template<typename T> bool getCalibration_Y(T &d, T &c, T &u)
        bint setCalibration_X(int, int, int)
        bint setCalibration_Y(int, int, int)
        bint setDeadZone(float)
        bint setAveraging(unsigned char)
        bint updateCalX()
        bint updateCalY()
        int getMinX()
        int getCenX()
        int getMaxX()
        int getMinY()
        int getCenY()
        int getMaxY()

#cdef extern from "iarduino_I2C_PI.h":
#    cdef cppclass iarduino_I2C:
#        bint readBytes(unsigned char, unsigned char, unsigned char*, unsigned char)



#!/usr/bin/python3
# ДАННЫЙ ПРИМЕР МЕНЯЕТ НАПРАВЛЕНИЕ ОСЕЙ:

from time import sleep
from curses import wrapper
# Подключаем библиотеку для работы с джойстиком I2C-flash.
from pyiArduinoI2Cjoystick import *

# Объявляем объект joy для работы с библиотекой, указывая адрес модуля на шине.
j = pyiArduinoI2Cjoystick(0x09)

# Если объявить объект без указания адреса, то адрес будет найден автоматически.
#j = pyiArduinoI2Cjoystick()

a, b, c = 0, 0, 0


#  Изменение направления оси X:

# Читаем установленные  калибровочные значения для оси X в переменные a,b,c.
a, b, c = j.getCalibration_X()

# Сохраняем прочитанные калибровочные значения для оси X,
# но указываем аргументы в обратном порядке: c, b, a.
j.setCalibration_X(c, b, a)


#  Изменение направления оси Y:                  #

# Читаем установленные  калибровочные значения для оси Y в переменные a,b,c.
a, b, c = j.getCalibration_Y()

# Сохраняем прочитанные калибровочные значения для оси Y,
# но указываем аргументы в обратном порядке: c, b, a.
j.setCalibration_Y(c, b, a)

def main(stdscr):

    stdscr.clear()

    print("Текущие координаты: (нажмите <ctrl+c> для выхода)", end="\r\n")

    # Получаем координаты
    x, y = j.getPosition()

    # Выводим  координаты джойстика.
    print("X:Y="+str(x)+":"+str(y))

    print("\r")

    sleep(.1)

    stdscr.refresh()

while True:
    wrapper(main)



#!/usr/bin/python3
# ДАННЫЙ ПРИМЕР ВЫВОДИТ КООРДИНАТЫ ДЖОЙСТИКА:

from time import sleep
from curses import wrapper

# Подключаем библиотеку для работы с джойстиком I2C-flash.
from pyiArduinoI2Cjoystick import *

# Объявляем объект для работы с библиотекой, указывая адрес модуля на шине.
j = pyiArduinoI2Cjoystick(0x09)

# Если объявить объект без указания адреса, то адрес будет найден автоматически.
#j = pyiArduinoI2Cjoystick()


def main(stdscr):

    stdscr.clear()

    print("Текущие координаты: (нажмите <ctrl+c> для выхода)", end="\r\n")

    # Получаем и выводим координаты по отдельности: #

    print("X=", end='')

    # Выводим координату по оси X, значение от -100 до 100.
    print(j.x, end='')

    print(", Y=", end='')

    # Выводим координату по оси Y, значение от -100 до 100.
    print(j.y, end='')

    # Получаем сразу обе координаты:
    x, y = j.getPosition()

    print(", X:Y=", end='')

    # Выводим координату по оси X, значение от -100 до 100.
    print(x, end='')

    print(":", end='')

    # Выводим координату по оси X, значение от -100 до 100.
    print(y, end="\r\n")

    sleep(.1)

    stdscr.refresh()

while True:

    wrapper(main)



from distutils.core import setup
from distutils.extension import Extension

def readme():
    with open('README.md') as readme:
        return readme.read()

setup(name='pyiArduinoI2Cjoystick',
    version='1.0.0',
    description='iarduino.ru module for Raspberry Pi',
    long_description=readme(),
    classifiers=[
        'Programming Language :: Python :: 3',
    ],
    url='http://github.com/tremaru/pyiArduinoI2Cjoystick',
    author='iarduino.ru',
    author_email='shop@iarduino.ru',
    license='MIT',
    ext_modules = [Extension(
        name="pyiArduinoI2Cjoystick",
        sources=["pyiArduinoI2Cjoystick/pyiArduinoI2Cjoystick.cpp"])],
#   include_package_data=True,
#   zip_safe=False,
#   python_requires='>=3',
)
